@@ -1,4 +1,7 @@
+//! Converts NAL packets to YUV images.
+
 use crate::error::NativeErrorExt;
+use crate::formats::YUVSource;
 use crate::Error;
 use openh264_sys2::{
     videoFormatI420, ISVCDecoder, ISVCDecoderVtbl, SBufferInfo, SDecodingParam, SParserBsInfo, SSysMEMBuffer, WelsCreateDecoder, WelsDestroyDecoder, DECODER_OPTION, DECODER_OPTION_ERROR_CON_IDC, DECODER_OPTION_NUM_OF_THREADS, DECODER_OPTION_TRACE_LEVEL, DECODING_STATE, WELS_LOG_DETAIL, WELS_LOG_QUIET
@@ -12,7 +15,7 @@ use std::ptr::{addr_of_mut, null, null_mut};
 #[rustfmt::skip]
 #[allow(non_snake_case)]
 #[derive(Debug)]
-struct DecoderRawAPI {
+pub struct DecoderRawAPI {
     decoder_ptr: *mut *const ISVCDecoderVtbl,
     initialize: unsafe extern "C" fn(arg1: *mut ISVCDecoder, pParam: *const SDecodingParam) -> c_long,
     uninitialize: unsafe extern "C" fn(arg1: *mut ISVCDecoder) -> c_long,
@@ -28,6 +31,7 @@ struct DecoderRawAPI {
 
 #[rustfmt::skip]
 #[allow(clippy::too_many_arguments)]
+#[allow(clippy::missing_safety_doc)]
 #[allow(non_snake_case)]
 #[allow(unused)]
 impl DecoderRawAPI {
@@ -43,59 +47,32 @@ impl DecoderRawAPI {
 
             Ok(DecoderRawAPI {
                 decoder_ptr,
-                initialize: (*(*decoder_ptr)).Initialize.ok_or(e())?,
-                uninitialize: (*(*decoder_ptr)).Uninitialize.ok_or(e())?,
-                decode_frame: (*(*decoder_ptr)).DecodeFrame.ok_or(e())?,
-                decode_frame_no_delay: (*(*decoder_ptr)).DecodeFrameNoDelay.ok_or(e())?,
-                decode_frame2: (*(*decoder_ptr)).DecodeFrame2.ok_or(e())?,
-                flush_frame: (*(*decoder_ptr)).FlushFrame.ok_or(e())?,
-                decode_parser: (*(*decoder_ptr)).DecodeParser.ok_or(e())?,
-                decode_frame_ex: (*(*decoder_ptr)).DecodeFrameEx.ok_or(e())?,
-                set_option: (*(*decoder_ptr)).SetOption.ok_or(e())?,
-                get_option: (*(*decoder_ptr)).GetOption.ok_or(e())?,
+                initialize: (*(*decoder_ptr)).Initialize.ok_or_else(e)?,
+                uninitialize: (*(*decoder_ptr)).Uninitialize.ok_or_else(e)?,
+                decode_frame: (*(*decoder_ptr)).DecodeFrame.ok_or_else(e)?,
+                decode_frame_no_delay: (*(*decoder_ptr)).DecodeFrameNoDelay.ok_or_else(e)?,
+                decode_frame2: (*(*decoder_ptr)).DecodeFrame2.ok_or_else(e)?,
+                flush_frame: (*(*decoder_ptr)).FlushFrame.ok_or_else(e)?,
+                decode_parser: (*(*decoder_ptr)).DecodeParser.ok_or_else(e)?,
+                decode_frame_ex: (*(*decoder_ptr)).DecodeFrameEx.ok_or_else(e)?,
+                set_option: (*(*decoder_ptr)).SetOption.ok_or_else(e)?,
+                get_option: (*(*decoder_ptr)).GetOption.ok_or_else(e)?,
             })
         }
     }
 
-    unsafe fn initialize(&self, pParam: *const SDecodingParam) -> c_long {
-        (self.initialize)(self.decoder_ptr, pParam)
-    }
+    // Exposing these will probably do more harm than good.
+    unsafe fn initialize(&self, pParam: *const SDecodingParam) -> c_long { (self.initialize)(self.decoder_ptr, pParam) }
+    unsafe fn uninitialize(&self, ) -> c_long { (self.uninitialize)(self.decoder_ptr) }
 
-    unsafe fn uninitialize(&self, ) -> c_long {
-        (self.uninitialize)(self.decoder_ptr)
-    }
-
-    unsafe fn decode_frame(&self, Src: *const c_uchar, iSrcLen: c_int, ppDst: *mut *mut c_uchar, pStride: *mut c_int, iWidth: *mut c_int, iHeight: *mut c_int) -> DECODING_STATE {
-        (self.decode_frame)(self.decoder_ptr, Src, iSrcLen, ppDst, pStride, iWidth, iHeight)
-    }
-
-    unsafe fn decode_frame_no_delay(&self, pSrc: *const c_uchar, iSrcLen: c_int, ppDst: *mut *mut c_uchar, pDstInfo: *mut SBufferInfo) -> DECODING_STATE {
-        (self.decode_frame_no_delay)(self.decoder_ptr, pSrc, iSrcLen, ppDst, pDstInfo)
-    }
-
-    unsafe fn decode_frame2(&self, pSrc: *const c_uchar, iSrcLen: c_int, ppDst: *mut *mut c_uchar, pDstInfo: *mut SBufferInfo) -> DECODING_STATE {
-        (self.decode_frame2)(self.decoder_ptr, pSrc, iSrcLen, ppDst, pDstInfo)
-    }
-
-    unsafe fn flush_frame(&self, ppDst: *mut *mut c_uchar, pDstInfo: *mut SBufferInfo) -> DECODING_STATE {
-        (self.flush_frame)(self.decoder_ptr, ppDst, pDstInfo)
-    }
-
-    unsafe fn decode_parser(&self, pSrc: *const c_uchar, iSrcLen: c_int, pDstInfo: *mut SParserBsInfo) -> DECODING_STATE {
-        (self.decode_parser)(self.decoder_ptr, pSrc, iSrcLen, pDstInfo)
-    }
-
-    unsafe fn decode_frame_ex(&self, pSrc: *const c_uchar, iSrcLen: c_int, pDst: *mut c_uchar, iDstStride: c_int, iDstLen: *mut c_int, iWidth: *mut c_int, iHeight: *mut c_int, iColorFormat: *mut c_int) -> DECODING_STATE {
-        (self.decode_frame_ex)(self.decoder_ptr, pSrc, iSrcLen, pDst, iDstStride, iDstLen, iWidth, iHeight, iColorFormat)
-    }
-
-    unsafe fn set_option(&self, eOptionId: DECODER_OPTION, pOption: *mut c_void) -> c_long {
-        (self.set_option)(self.decoder_ptr, eOptionId, pOption)
-    }
-
-    unsafe fn get_option(&self, eOptionId: DECODER_OPTION, pOption: *mut c_void) -> c_long {
-        (self.get_option)(self.decoder_ptr, eOptionId, pOption)
-    }
+    pub unsafe fn decode_frame(&self, Src: *const c_uchar, iSrcLen: c_int, ppDst: *mut *mut c_uchar, pStride: *mut c_int, iWidth: *mut c_int, iHeight: *mut c_int) -> DECODING_STATE { (self.decode_frame)(self.decoder_ptr, Src, iSrcLen, ppDst, pStride, iWidth, iHeight) }
+    pub unsafe fn decode_frame_no_delay(&self, pSrc: *const c_uchar, iSrcLen: c_int, ppDst: *mut *mut c_uchar, pDstInfo: *mut SBufferInfo) -> DECODING_STATE { (self.decode_frame_no_delay)(self.decoder_ptr, pSrc, iSrcLen, ppDst, pDstInfo) }
+    pub unsafe fn decode_frame2(&self, pSrc: *const c_uchar, iSrcLen: c_int, ppDst: *mut *mut c_uchar, pDstInfo: *mut SBufferInfo) -> DECODING_STATE { (self.decode_frame2)(self.decoder_ptr, pSrc, iSrcLen, ppDst, pDstInfo) }
+    pub unsafe fn flush_frame(&self, ppDst: *mut *mut c_uchar, pDstInfo: *mut SBufferInfo) -> DECODING_STATE { (self.flush_frame)(self.decoder_ptr, ppDst, pDstInfo) }
+    pub unsafe fn decode_parser(&self, pSrc: *const c_uchar, iSrcLen: c_int, pDstInfo: *mut SParserBsInfo) -> DECODING_STATE { (self.decode_parser)(self.decoder_ptr, pSrc, iSrcLen, pDstInfo) }
+    pub unsafe fn decode_frame_ex(&self, pSrc: *const c_uchar, iSrcLen: c_int, pDst: *mut c_uchar, iDstStride: c_int, iDstLen: *mut c_int, iWidth: *mut c_int, iHeight: *mut c_int, iColorFormat: *mut c_int) -> DECODING_STATE { (self.decode_frame_ex)(self.decoder_ptr, pSrc, iSrcLen, pDst, iDstStride, iDstLen, iWidth, iHeight, iColorFormat) }
+    pub unsafe fn set_option(&self, eOptionId: DECODER_OPTION, pOption: *mut c_void) -> c_long {  (self.set_option)(self.decoder_ptr, eOptionId, pOption) }
+    pub unsafe fn get_option(&self, eOptionId: DECODER_OPTION, pOption: *mut c_void) -> c_long { (self.get_option)(self.decoder_ptr, eOptionId, pOption) }
 }
 
 impl Drop for DecoderRawAPI {
@@ -113,9 +90,9 @@ impl Drop for DecoderRawAPI {
 #[derive(Default, Copy, Clone, Debug)]
 pub struct DecoderConfig {
     params: SDecodingParam,
-    num_threads: i32,
-    debug: i32,
-    error_concealment: i32,
+    num_threads: DECODER_OPTION,
+    debug: DECODER_OPTION,
+    error_concealment: DECODER_OPTION,
 }
 
 impl DecoderConfig {
@@ -133,20 +110,21 @@ impl DecoderConfig {
     ///
     /// # Safety
     ///
-    /// Right now it seems to be unclear if setting a thread count is entirely safe.
-    /// If you have proof either way, please file an PR removing the `unsafe` marker, or this section.
+    /// This setting is only supported on some platforms and is known to segfault on others. You should consult with
+    /// the upstream OpenH264 project where and when it is safe to set this.
     pub unsafe fn num_threads(mut self, num_threads: u32) -> Self {
         self.num_threads = num_threads as i32;
         self
     }
 
+    /// Enables detailed console logging inside OpenH264.
     pub fn debug(mut self, value: bool) -> Self {
         self.debug = if value { WELS_LOG_DETAIL } else { WELS_LOG_QUIET };
         self
     }
 }
 
-/// An [OpenH264](https://github.com/cisco/openh264) decoder, converts packets to YUV.
+/// An [OpenH264](https://github.com/cisco/openh264) decoder.
 #[derive(Debug)]
 pub struct Decoder {
     raw_api: DecoderRawAPI,
@@ -175,7 +153,7 @@ impl Decoder {
         Ok(Self { raw_api: raw })
     }
 
-    /// Decodes a complete H.264 bitstream and returns the latest picture.
+    /// Decodes a series of H.264 NAL packets and returns the latest picture.
     ///
     /// This function can be called with:
     ///
@@ -188,7 +166,7 @@ impl Decoder {
     /// # Errors
     ///
     /// The function returns and error if any of the packets is incomplete, e.g., was truncated.
-    pub fn decode_no_delay(&mut self, packet: &[u8]) -> Result<DecodedYUV, Error> {
+    pub fn decode(&mut self, packet: &[u8]) -> Result<DecodedYUV, Error> {
         let mut dst = [null_mut(); 3];
         let mut buffer_info = SBufferInfo::default();
 
@@ -211,6 +189,35 @@ impl Decoder {
             Ok(DecodedYUV { info, y, u, v })
         }
     }
+
+    /// Obtain the raw API for advanced use cases.
+    ///
+    /// When resorting to this call, please consider filing an issue / PR.
+    ///
+    /// # Safety
+    ///
+    /// You must not set parameters the decoder relies on, we recommend checking the source.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use openh264::decoder::{DecoderConfig, Decoder};
+    ///
+    /// # use openh264::Error;
+    /// #
+    /// # fn try_main() -> Result<(), Error> {
+    /// let config = DecoderConfig::default();
+    /// let mut decoder = Decoder::with_config(config)?;
+    ///
+    /// unsafe {
+    ///     let _ = decoder.raw_api();
+    /// };
+    /// # Ok(())
+    /// # }
+    /// ```
+    pub unsafe fn raw_api(&mut self) -> &mut DecoderRawAPI {
+        &mut self.raw_api
+    }
 }
 
 impl Drop for Decoder {
@@ -288,7 +295,12 @@ impl<'a> DecodedYUV<'a> {
         )
     }
 
+    // TODO: Ideally we'd like to move these out into a converter in `formats`.
     /// Writes the image into a byte buffer of size `w*h*3`.
+    ///
+    /// # Panics
+    ///
+    /// Panics if the target image dimension don't match the configured format.
     pub fn write_rgb8(&self, target: &mut [u8]) -> Result<(), Error> {
         let dim = self.dimension_rgb();
         let strides = self.strides_yuv();
@@ -329,6 +341,12 @@ impl<'a> DecodedYUV<'a> {
         Ok(())
     }
 
+    // TODO: Ideally we'd like to move these out into a converter in `formats`.
+    /// Writes the image into a byte buffer of size `w*h*4`.
+    ///
+    /// # Panics
+    ///
+    /// Panics if the target image dimension don't match the configured format.
     pub fn write_rgba8(&self, target: &mut [u8]) -> Result<(), Error> {
         let dim = self.dimension_rgb();
         let strides = self.strides_yuv();
@@ -370,3 +388,38 @@ impl<'a> DecodedYUV<'a> {
         Ok(())
     }
 }
+
+#[cfg(feature = "encoder")]
+impl<'a> YUVSource for DecodedYUV<'a> {
+    fn width(&self) -> i32 {
+        self.info.iWidth
+    }
+
+    fn height(&self) -> i32 {
+        self.info.iHeight
+    }
+
+    fn y(&self) -> &[u8] {
+        self.y
+    }
+
+    fn u(&self) -> &[u8] {
+        self.u
+    }
+
+    fn v(&self) -> &[u8] {
+        self.v
+    }
+
+    fn y_stride(&self) -> i32 {
+        self.info.iStride[0]
+    }
+
+    fn u_stride(&self) -> i32 {
+        self.info.iStride[1]
+    }
+
+    fn v_stride(&self) -> i32 {
+        self.info.iStride[1]
+    }
+}
