@@ -1,4 +1,5 @@
 #![cfg_attr(feature = "backtrace", feature(backtrace))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 //!
 //! [![Latest Version]][crates.io]
 //! [![docs]][docs.rs]
@@ -8,19 +9,16 @@
 //!
 //! # OpenH264 Rust API
 //!
-//! Idiomatic and low-level bindings for [OpenH264](https://github.com/cisco/openh264), converting<sup>*</sup> between these two in Rust:
+//! Idiomatic and low-level bindings for [OpenH264](https://github.com/cisco/openh264), converting between these two in Rust:
 //!
 //! ![sample_image](https://media.githubusercontent.com/media/ralfbiedert/openh264-rust/master/gfx/title2.jpg)
 //!
 //!
-//! *High-level wrapped decoder only for now, encoder PRs welcome.
-//!
 //! ## Example API
 //!
-//! Here we convert the last image of a H264 stream to a RGB byte array.
-//!
+//! **Decode** some H.264 bitstream to YUV:
 //! ```rust
-//! use openh264::Decoder;
+//! use openh264::decoder::Decoder;
 //! # use openh264::Error;
 //!
 //! # fn main() -> Result<(), Error> {
@@ -28,8 +26,30 @@
 //! let mut rgb_out = [0; 512 * 512 * 3];
 //! let h264_in = include_bytes!("../tests/data/multi_512x512.h264");
 //!
-//! // Decode to YUV, then convert and write RGB.
-//! decoder.decode_no_delay(&h264_in[..])?.write_rgb8(&mut rgb_out)?;
+//! // Decode H.264 bitstream to YUV.
+//! let yuv = decoder.decode(&h264_in[..])?;
+//! # Ok(())
+//! # }
+//!
+//! ```
+//!
+//!
+//! And **encode** the same YUV back to H.264:
+//! ```rust
+//! # use openh264::decoder::Decoder;
+//! # use openh264::Error;
+//! use openh264::encoder::{Encoder, EncoderConfig};
+//! # fn main() -> Result<(), Error> {
+//! # let mut decoder = Decoder::new()?;
+//! # let mut rgb_out = [0; 512 * 512 * 3];
+//! # let h264_in = include_bytes!("../tests/data/multi_512x512.h264");
+//! # let yuv = decoder.decode(&h264_in[..])?;
+//!
+//! let config = EncoderConfig::new(512, 512);
+//! let mut encoder = Encoder::with_config(config)?;
+//!
+//! // Encode YUV back into H.264.
+//! let bitstream = encoder.encode(&yuv)?;
 //! # Ok(())
 //! # }
 //!
@@ -51,7 +71,7 @@
 //! 🆗 the usual shenanigans required;
 //! ❌ not supported.
 //!
-//! <sup>1</sup> via `cargo build --target <platform>`, [needs `CXX` set](https://cheats.rs/#cross-compilation) <br/>
+//! <sup>1</sup> via `cargo build --target <platform>`, [needs `CXX` set](https://cheats.rs/#cross-compilation) and `libc++_shared.so`. <br/>
 //! <sup>2</sup> unclear if could ever work, investigation welcome
 //!
 //!
@@ -60,18 +80,19 @@
 //! Tested on a i9-9900K, Windows 10, single threaded decoding:
 //!
 //! ```text
+//! -- Decoder --
 //! test decode_yuv_single_1920x1080     ... bench:   9,243,380 ns/iter (+/- 497,200)
 //! test decode_yuv_single_512x512_cabac ... bench:   1,841,775 ns/iter (+/- 53,211)
 //! test decode_yuv_single_512x512_cavlc ... bench:   2,076,030 ns/iter (+/- 7,287)
 //! test whole_decoder                   ... bench:   2,874,107 ns/iter (+/- 62,643)
 //!
-//! test result: ok. 0 passed; 0 failed; 0 ignored; 5 measured; 0 filtered out; finished in 14.26s
+//! -- Encoder --
+//! test encode_1920x1080_from_yuv       ... bench:  38,657,620 ns/iter (+/- 793,310)
+//! test encode_512x512_from_yuv         ... bench:   6,420,605 ns/iter (+/- 1,003,485)
 //!
-//! Running unittests (target\release\deps\yuv2rgb-5a3aaabbb6bf3e8a.exe)
-//!
-//! running 2 tests
-//! test convert_yuv_to_rgb_1920x1080 ... bench:   7,226,290 ns/iter (+/- 110,871)
-//! test convert_yuv_to_rgb_512x512   ... bench:     907,340 ns/iter (+/- 28,296)
+//! -- Color Conversion --
+//! test convert_yuv_to_rgb_1920x1080    ... bench:   7,226,290 ns/iter (+/- 110,871)
+//! test convert_yuv_to_rgb_512x512      ... bench:     907,340 ns/iter (+/- 28,296)
 //! ```
 //!
 //! If you want to improve these numbers you can submit PRs that
@@ -81,6 +102,8 @@
 //!
 //! ## Compile Features
 //!
+//! - `decoder` - Enable the decoder. Used by default.
+//! - `encoder` - Enable the encoder. Used by default.
 //! - `backtrace` - Enable backtraces on errors (requires nightly)
 //!
 //! ## FAQ
@@ -126,7 +149,7 @@
 //!
 //! Especially needed:
 //!
-//! - [ ] Encoder wrapper
+//! - [ ] BT.601 / BT.709 YUV <-> RGB Conversion
 //! - [ ] Enabling of platform specific assembly (without breaking or complicating build)
 //! - [ ] Faster YUV to RGB conversion
 //! - [ ] Have script to automatically update / import OpenH264 source (or submodule?)
@@ -146,8 +169,15 @@
 //! [docs]: https://docs.rs/openh264/badge.svg
 //! [docs.rs]: https://docs.rs/openh264/
 
-mod decoder;
 mod error;
+pub mod formats;
+
+#[cfg(feature = "decoder")]
+#[cfg_attr(docsrs, doc(cfg(feature = "decoder")))]
+pub mod decoder;
+
+#[cfg(feature = "encoder")]
+#[cfg_attr(docsrs, doc(cfg(feature = "encoder")))]
+pub mod encoder;
 
-pub use decoder::{DecodedYUV, Decoder, DecoderConfig};
 pub use error::Error;
