@@ -1,4 +1,7 @@
-use openh264::{Decoder, DecoderConfig, Error};
+#![cfg(feature = "decoder")]
+
+use openh264::decoder::{Decoder, DecoderConfig};
+use openh264::Error;
 
 #[test]
 fn can_get_decoder() -> Result<(), Error> {
@@ -8,6 +11,18 @@ fn can_get_decoder() -> Result<(), Error> {
     Ok(())
 }
 
+#[test]
+fn can_access_raw_api() -> Result<(), Error> {
+    let config = DecoderConfig::default();
+    let mut decoder = Decoder::with_config(config)?;
+
+    unsafe {
+        let _ = decoder.raw_api();
+    };
+
+    Ok(())
+}
+
 #[test]
 #[rustfmt::skip]
 fn can_decode_single() -> Result<(), Error> {
@@ -21,7 +36,7 @@ fn can_decode_single() -> Result<(), Error> {
         let config = DecoderConfig::default().debug(false);
         let mut decoder = Decoder::with_config(config)?;
 
-        let yuv = decoder.decode_no_delay(src)?;
+        let yuv = decoder.decode(src)?;
 
         let dim = yuv.dimension_rgb();
         let rgb_len = dim.0 * dim.1 * 3;
@@ -37,12 +52,10 @@ fn can_decode_single() -> Result<(), Error> {
 fn can_decode_multi_to_end() -> Result<(), Error> {
     let src = &include_bytes!("data/multi_512x512.h264")[..];
 
-    println!("BYTE ARRAY LENGTH {}", src.len());
-
     let config = DecoderConfig::default().debug(false);
     let mut decoder = Decoder::with_config(config)?;
 
-    decoder.decode_no_delay(src)?;
+    decoder.decode(src)?;
 
     Ok(())
 }
@@ -59,7 +72,7 @@ fn can_decode_multi_by_step() -> Result<(), Error> {
     let mut p = 0;
 
     for l in packet_lengths {
-        decoder.decode_no_delay(&src[p..p + l])?;
+        decoder.decode(&src[p..p + l])?;
 
         p += l;
     }
@@ -74,7 +87,31 @@ fn fails_on_truncated() -> Result<(), Error> {
     let config = DecoderConfig::default().debug(false);
     let mut decoder = Decoder::with_config(config)?;
 
-    assert!(decoder.decode_no_delay(src).is_err());
+    assert!(decoder.decode(src).is_err());
+
+    Ok(())
+}
+
+#[test]
+#[cfg(feature = "encoder")]
+fn what_goes_around_comes_around() -> Result<(), Error> {
+    use openh264::encoder::{Encoder, EncoderConfig};
+    use openh264::formats::RBGYUVConverter;
+
+    let src = &include_bytes!("data/lenna_128x128.rgb")[..];
+
+    let config = EncoderConfig::new(128, 128);
+    let mut encoder = Encoder::with_config(config)?;
+    let mut converter = RBGYUVConverter::new(128, 128);
+
+    converter.convert(src);
+
+    let stream = encoder.encode(&converter)?;
+
+    let src = stream.to_vec();
+    let config = DecoderConfig::default();
+    let mut decoder = Decoder::with_config(config)?;
+    decoder.decode(&src)?;
 
     Ok(())
 }
